package net.greenfieldtech.cloudonixsdk.cloudonixdialer.utils;

import android.content.Context;
import android.media.AudioManager;
import android.media.Ringtone;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import android.os.Vibrator;
import android.util.Log;

public class Ringer {

  private static final String TAG = "Ringer";

  private static final int VIBRATE_MS = 1000;
  private static final int PAUSE_MS = 1000;

  Vibrator mVibrator;
  VibratorThread mVibratorThread;
  HandlerThread mRingerThread;
  Context context;

  private RingWorkerHandler ringerWorker;

  public Ringer(Context aContext) {
    context = aContext;
    mVibrator = (Vibrator) context.getSystemService(Context.VIBRATOR_SERVICE);
    mRingerThread = new HandlerThread("RingerThread");
    mRingerThread.start();
    ringerWorker = new RingWorkerHandler(mRingerThread.getLooper());

  }

  public void startRinging() {

    synchronized (this) {

      AudioManager audioManager =
          (AudioManager) context.getSystemService(Context.AUDIO_SERVICE);

      Ringtone ringtone = getDefaultRingtone();
      ringerWorker.setRingtone(ringtone);

      // Silent mode
      int ringerMode = audioManager.getRingerMode();
      if (ringerMode == AudioManager.RINGER_MODE_SILENT) {
        Log.d(TAG, "Silent mode - no ringing will be played");
        return;
      }

      // Vibrate mode
      int vibrateSetting = audioManager.getVibrateSetting(AudioManager.VIBRATE_TYPE_RINGER);
      if (mVibratorThread == null &&
          (vibrateSetting == AudioManager.VIBRATE_SETTING_ON ||
              ringerMode == AudioManager.RINGER_MODE_VIBRATE)) {
        mVibratorThread = new VibratorThread();
        mVibratorThread.start();
      }

      // Vibrate only
      if (ringerMode == AudioManager.RINGER_MODE_VIBRATE ||
          audioManager.getStreamVolume(AudioManager.STREAM_RING) == 0) {
        Log.d(TAG, "Vibrate / no volume mode - no ringing will be played");
        return;
      }

      if (ringtone == null) {
        Log.d(TAG, "No ringtone available - skipping");
        return;
      }

      ringerWorker.startRinging(audioManager);

    }
  }

  public boolean isRinging() {
    return (!ringerWorker.isStopped() || mVibratorThread != null);
  }

  public void stopRinging() {
    synchronized (this) {
      stopVibrator();
      stopRingerWorker();
    }
  }


  private void stopRingerWorker() {
    ringerWorker.setStopFlag();
  }

  private void stopVibrator() {

    if (mVibratorThread != null) {
      mVibratorThread.interrupt();
      try {
        mVibratorThread.join(250);
      } catch (InterruptedException e) {
      }
      mVibratorThread = null;
    }
  }

  public void updateRingerMode() {

    AudioManager audioManager = (AudioManager) context.getSystemService(Context.AUDIO_SERVICE);
    synchronized (this) {
      int ringerMode = audioManager.getRingerMode();

      // Silent
      if (ringerMode == AudioManager.RINGER_MODE_SILENT) {
        stopRinging();
        return;
      }

      // Vibrate
      int vibrateSetting = audioManager.getVibrateSetting(AudioManager.VIBRATE_TYPE_RINGER);
      // If not already started restart it
      if (mVibratorThread == null && (vibrateSetting == AudioManager.VIBRATE_SETTING_ON
          || ringerMode == AudioManager.RINGER_MODE_VIBRATE)) {
        mVibratorThread = new VibratorThread();
        mVibratorThread.start();
      }

      // Vibrate only
      if (ringerMode == AudioManager.RINGER_MODE_VIBRATE
          || audioManager.getStreamVolume(AudioManager.STREAM_RING) == 0) {
        stopRingerWorker();
        return;
      }

      // Normal ringer
      ringerWorker.startRinging(audioManager);
    }
  }

  private class VibratorThread extends Thread {

    public void run() {
      try {
        while (true) {
          mVibrator.vibrate(VIBRATE_MS);
          Thread.sleep(VIBRATE_MS + PAUSE_MS);
        }
      } catch (InterruptedException ex) {
        Log.d(TAG, "Vibrator thread interrupted");
      } finally {
        mVibrator.cancel();
      }
      Log.d(TAG, "Vibrator thread exiting");
    }
  }

  private class RingWorkerHandler extends Handler {

    public static final int PROGRESS_RING = 0;
    private Boolean mShallStop = false;
    private Ringtone mRingtone = null;

    public RingWorkerHandler(Looper looper) {
      super(looper);
    }

    public void startRinging(AudioManager audioManager) {
      if (mRingtone != null) {
        Message msg = ringerWorker.obtainMessage(RingWorkerHandler.PROGRESS_RING);
        msg.arg1 = RingWorkerHandler.PROGRESS_RING;
        audioManager.setMode(AudioManager.MODE_RINGTONE);
        ringerWorker.sendMessage(msg);
      }
    }

    public synchronized void setRingtone(Ringtone ringtone) {
      if (this.mRingtone != null) {
        try {
          this.mRingtone.stop();
        } catch (NullPointerException ex) {

        }
      }
      this.mRingtone = ringtone;
      mShallStop = false;
    }

    public synchronized void setStopFlag() {
      mShallStop = true;
    }

    public synchronized boolean isStopped() {
      return mShallStop || (mRingtone == null);
    }

    public void handleMessage(Message msg) {
      try {
        if (mRingtone == null) {
          return;
        }
        if (msg.arg1 == PROGRESS_RING) {
          synchronized (mShallStop) {
            if (mShallStop) {
              mRingtone.stop();
              mRingtone = null;
              return;
            }
          }

          if (!mRingtone.isPlaying()) {
            mRingtone.play();
          }

          Message msgBis = ringerWorker.obtainMessage(RingWorkerHandler.PROGRESS_RING);
          msg.arg1 = RingWorkerHandler.PROGRESS_RING;
          ringerWorker.sendMessageDelayed(msgBis, 100);
        }
      } catch (NullPointerException ex) {
      }


    }
  }

  private Ringtone getDefaultRingtone() {

    Uri ringtoneURI = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_RINGTONE);

    return RingtoneManager.getRingtone(context, ringtoneURI);
  }
}
